﻿using System;
using System.ComponentModel;
using System.Reflection;

namespace Inedo.Consolation
{
    internal sealed class CommandConsoleOption : ConsoleOption
    {
        private ParameterInfo parameter;
        private object currentValue;

        internal CommandConsoleOption(ParameterInfo parameter)
            : base(parameter)
        {
            this.parameter = parameter;
            this.IsFlag = parameter.ParameterType == typeof(bool) || parameter.ParameterType == typeof(bool?);
            if (parameter.HasDefaultValue)
                this.DefaultValue = parameter.DefaultValue;

            this.IsPositional = Attribute.IsDefined(parameter, typeof(PositionalArgumentAttribute));
            this.IsRequired = Attribute.IsDefined(parameter, typeof(RequiredValueAttribute));
            var browsableAttr = (BrowsableAttribute)Attribute.GetCustomAttribute(parameter, typeof(BrowsableAttribute));
            if (browsableAttr != null)
                this.IsBrowsable = browsableAttr.Browsable;
            else
                this.IsBrowsable = true;
        }

        public override object CurrentValue => this.currentValue;
        public bool IsPositional { get; }
        public bool IsRequired { get; }
        public bool IsBrowsable { get; }

        public override void SetValue(string valueText)
        {
            this.currentValue = this.ParseValue(valueText, this.parameter.ParameterType);
        }
        public override void SetFlag(bool enableDisable)
        {
            this.currentValue = enableDisable;
        }
    }
}
